/*
    reversi - play a game of reversi against the computer or a human
    Copyright (C) 1992  Elias Martenson

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact me by email at elias@omicron.se
*/

#include <stdio.h>
#include <signal.h>
#include <string.h>
#include <assert.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Xaw/Command.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/Sme.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/Cardinals.h>	

#include "reversi.h"

char *getenv( char * );

int game_board2[64];
int *game_board;
int *old_gameboard;
int old_pointer;
int to_load = 0;
int play_computer[2];
int p1, p2;
int game_l[2];
 
/* Global variables dealing with the X-interface */
String fallback_resources[] = {
  "*gameForm.width: 576",
  "*gameForm.height: 576",
  "*quitGame.label: Quit",
  "*saveGame.label: Save",
  "*showPossible.label: Show possible moves",
  "*undoMove.label: Undo move",
  "*newGame.label: New game",
  "*message.label: Welcome to Xreversi!",
  "*message.width: 560",
  "*levelMenu1Item0.label: Human",
  "*levelMenu1Item1.label: Apprentice",
  "*levelMenu1Item2.label: Amateur",
  "*levelMenu1Item3.label: Regular",
  "*levelMenu1Item4.label: Advanced",
  "*levelMenu1Item5.label: Hard",
  "*levelMenu1Item6.label: Very Hard",
  "*levelMenu2Item0.label: Human",
  "*levelMenu2Item1.label: Apprentice",
  "*levelMenu2Item2.label: Amateur",
  "*levelMenu2Item3.label: Regular",
  "*levelMenu2Item4.label: Advanced",
  "*levelMenu2Item5.label: Hard",
  "*levelMenu2Item6.label: Very Hard",
  "*blank.height: 20",
  NULL };

int to_play;
int comp_think = FALSE;
Widget message;
Widget currentlyPlaying;
Widget levelMenuCommand1, levelMenuCommand2;
Widget gameForm;
int showing_possible = FALSE;
int in_play = FALSE;
int levels[] = { 0, 1, 2, 3, 4, 5, 6, 7 };
char *level_name[] = {
  "Apprentice",
  "Amateur",
  "Regular",
  "Advanced",
  "Hard",
  "Very Hard"
};

Screen *screen;
Display *display;
Window win;
GC dgc;
Visual *dv;
int window_width, window_height;
XWindowAttributes win_attr;
XtAppContext app_con;


#ifdef HAVE_VOID_SIGNALS
void exit_rout();
#else
int exit_rout();
#endif

int do_x_move( int, int, int );
void do_show_possible( Widget, XtPointer, XtPointer );
void do_undo_move( Widget, XtPointer, XtPointer );
void xsave_game( Widget, XtPointer, XtPointer );
void redraw_board( void );
void draw_board();
void handle_event( Widget, XtPointer, XEvent * );
void draw_tile( int, int, int, int, int, int, int );
void new_game( Widget, XtPointer, XtPointer );
void select_level1( Widget, XtPointer, XtPointer );
void select_level2( Widget, XtPointer, XtPointer );
void update_message( char * );
void update_playing( void );
void update_levels( void );

main( argc, argv )
int argc;
char **argv;
{
  int w_b;
  int starting_player;
  char tmp_str[80], tmp[80];
  FILE *fp;
  int curr_scr;
  int x, y;
  int t;
  int cx, cy;
  int c;
  Widget toplevel, basePaned, quitGame, menuBox,
         saveGame, showPossible, undoMove, newGame,
         levelMenu1, levelMenu2;
  Widget levelEntry1[8];
  Widget levelEntry2[8];
  Widget b1, b2;
  Arg args[10];


/* Init X */
    toplevel = XtAppInitialize(&app_con, "Xreversi",
			       NULL, ZERO,
			       &argc, argv, fallback_resources,
			       NULL, ZERO);



  if( argc == 2 ){
    if( strcmp( argv[1], "-r" ) == 0 ){
      to_load = TRUE;
    }
    else{
      usage();
    }
  }
  else if( argc > 2 ){
    usage();
  }
  
  old_pointer = 0;
  starting_player = 0;



  basePaned = XtCreateManagedWidget( "basePaned", panedWidgetClass,
				    toplevel, NULL, ZERO );
  menuBox = XtCreateManagedWidget( "menuBox", boxWidgetClass,
				  basePaned, NULL, ZERO );
  quitGame = XtCreateManagedWidget( "quitGame", commandWidgetClass,
				   menuBox, NULL, ZERO );
  saveGame = XtCreateManagedWidget( "saveGame", commandWidgetClass,
				   menuBox, NULL, ZERO );
  undoMove = XtCreateManagedWidget( "undoMove", commandWidgetClass,
				   menuBox, NULL, ZERO );
  showPossible = XtCreateManagedWidget( "showPossible", commandWidgetClass,
				       menuBox, NULL, ZERO );
  newGame = XtCreateManagedWidget( "newGame", commandWidgetClass,
				  menuBox, NULL, ZERO );
  gameForm = XtCreateManagedWidget( "gameForm", widgetClass,
				   basePaned, NULL, ZERO );

  levelMenuCommand1 = XtCreateManagedWidget( "levelMenuCommand1", menuButtonWidgetClass,
					    menuBox, NULL, ZERO );
  levelMenu1 = XtCreatePopupShell( "menu", simpleMenuWidgetClass,
				  levelMenuCommand1, NULL, ZERO );
  for( c = 0 ; c < 7 ; c++ ){
    sprintf( tmp, "levelMenu1Item%d", c );

    levelEntry1[c] = XtCreateManagedWidget( tmp, smeBSBObjectClass,
					   levelMenu1, NULL, ZERO );

    XtAddCallback( levelEntry1[c], XtNcallback, select_level1, levels + c );

    if( c == 0 ){
      b1 = XtCreateManagedWidget( "blank", smeObjectClass,
					     levelMenu1, NULL, ZERO );
    }

  }

  levelMenuCommand2 = XtCreateManagedWidget( "levelMenuCommand2", menuButtonWidgetClass,
					    menuBox, NULL, ZERO );
  levelMenu2 = XtCreatePopupShell( "menu", simpleMenuWidgetClass,
				  levelMenuCommand2, NULL, ZERO );
  for( c = 0 ; c < 7 ; c++ ){
    sprintf( tmp, "levelMenu2Item%d", c );

    levelEntry2[c] = XtCreateManagedWidget( tmp, smeBSBObjectClass,
					   levelMenu2, NULL, ZERO );
    XtAddCallback( levelEntry2[c], XtNcallback, select_level2, levels + c );
   
    if( c == 0 ){
      b2 = XtCreateManagedWidget( "blank", smeObjectClass,
					     levelMenu2, NULL, ZERO );
    }

  }

  message = XtCreateManagedWidget( "message", labelWidgetClass,
				  menuBox, NULL, ZERO );
  currentlyPlaying = XtCreateManagedWidget( "currentlyPlaying", labelWidgetClass,
				  menuBox, NULL, ZERO );

  XtAddCallback( quitGame, XtNcallback, exit_rout, NULL );
  XtAddCallback( showPossible, XtNcallback, do_show_possible, NULL );
  XtAddCallback( undoMove, XtNcallback, do_undo_move, NULL );
  XtAddCallback( saveGame, XtNcallback, xsave_game, NULL );
  XtAddCallback( newGame, XtNcallback, new_game, NULL );

  game_board = (int *)game_board2;
  if( (old_gameboard = (int *)malloc( sizeof(int) * 64 * 60 )) == NULL ){
    fprintf( stderr, "Could not allocate memory.\n" );
    exit( 1 );
  }

  if( !to_load ){
    
    w_b = 0;
    play_computer[0] = 0;
    play_computer[1] = 0;
    p1 = 0;
    p2 = 0;
    game_l[0] = 0;
    game_l[1] = 0;

    update_levels();
    
    for( cx = 0 ; cx < 8 ; cx++ ){
      for( cy = 0 ; cy < 8 ; cy++ ){
	get_board( game_board, cx, cy ) = 0;
      }
    }

    for( cx = 0 ; cx < 8 ; cx++ ){
      for( cy = 0 ; cy < 8 ; cy++ ){
	get_board( old_gameboard, cx, cy ) = get_board( game_board, cx, cy );
      }
    }
    
    old_pointer += 64;
    
    for( cx = 0 ; cx < 8 ; cx++ ){
      for( cy = 0 ; cy < 8 ; cy++ ){
	get_board( old_gameboard + old_pointer, cx, cy ) = get_board( game_board, cx, cy );
      }
    }
    
    old_pointer += 64;

  }
  else{
    strcpy( tmp_str, getenv( "HOME" ) );
    strcat( tmp_str, "/.reversi_save" );
    if( (fp = fopen( tmp_str, "r" )) == NULL ){
      fprintf( stderr, "Could not open save file\n" );
      exit_rout( 0 );
    }
    fread( &starting_player, sizeof( int ), 1, fp );
    fread( &(play_computer[0]), sizeof( int ), 1, fp );
    fread( &(play_computer[1]), sizeof( int ), 1, fp );
    fread( &(game_l[0]), sizeof( int ), 1, fp );
    fread( &(game_l[1]), sizeof( int ), 1, fp );

    curr_scr = 0;
    do{
      for( x = 0 ; x < 8 ; x++ ){
	for( y = 0 ; y < 8 ; y++ ){
	  fread( &t, sizeof( int ), 1, fp );
	  get_board( old_gameboard + curr_scr, x, y ) = t;
	}
      }
      curr_scr += 64;
    }while( !feof( fp ) );
    fclose( fp );
    curr_scr -= 64;
    old_pointer = curr_scr;
    for( x = 0 ; x < 8 ; x++ ){
      for( y = 0 ; y < 8 ; y++ ){
	get_board( game_board, x, y ) = get_board( old_gameboard + curr_scr - 64, x, y );
      }
    }

    in_play = TRUE;
  }

  to_play = starting_player;

  XtRealizeWidget(toplevel);

  display = XtDisplay( toplevel );
  screen = XtScreen( toplevel );
  win = XtWindow( gameForm );
  dgc = screen->default_gc;
  dv = screen->root_visual;

  XClearWindow( display, win );

  XGetWindowAttributes(display, win, &win_attr);
  window_width = win_attr.width;
  window_height = win_attr.height;

  XtAddEventHandler( gameForm, ExposureMask | StructureNotifyMask | ButtonPressMask,
		    FALSE, handle_event, NULL );

  update_playing();

  update_playing();

  redraw_board();

  XtAppMainLoop(app_con);

  exit( 0 );
}

#ifdef HAVE_VOID_SIGNALS
void exit_rout( sig )
#else
int exit_rout( sig )
#endif
int sig;
{
  exit(0);
}

int do_x_move( int x, int y, int comp_move )
{
  int cx, cy;
  int get_x, get_y;
  Arg args[1];
  XEvent event;

  if( !in_play ){
    update_message( "There is no game in progress right now" );
    return 0;
  }

  /* Currently always FALSE */
  if( !comp_think ){
    if( possible_move_square( game_board, x, y, to_play ) || comp_move ){
      if( comp_move ){
	comp_think = TRUE;
	update_message( "Computer is thinking, please wait" );
	XtAppNextEvent( app_con, &event );
	handle_event( gameForm, NULL, &event );
	get_comp_move( to_play, game_board, &get_x, &get_y, game_l[to_play], 1 );
	update_message( "Finished thinking" );
	put_square_d( to_play, game_board, get_x, get_y );
	comp_think = FALSE;
      }
      else{
	put_square_d( to_play, game_board, x, y );
      }

      if( showing_possible ){
	redraw_board();
	showing_possible = FALSE;
      }

      for( cx = 0 ; cx < 8 ; cx++ ){
	for( cy = 0 ; cy < 8 ; cy++ ){
	  get_board( old_gameboard + old_pointer, cx, cy ) = get_board( game_board, cx, cy );
	}
      }
      
      old_pointer += 64;
      
      to_play = !to_play;
      if( !possible_move( game_board, to_play ) ){
	
	if( !possible_move( game_board, !to_play ) ){
	  win_game();
	  return 0;
	}
	
	to_play = !to_play;
	
	update_message( "No possible moves, make another move" );
      }
      
      update_playing();

      if( play_computer[to_play] == 1 ){
	return 1;
      }

    }
  }
  else{
    update_message( "Sorry, not yet. The computer is thinking." );
  }

  return 0;
}

void do_undo_move( Widget a, XtPointer b, XtPointer c )
{
  int count_x, count_y;

  if( in_play ){
    if( old_pointer > 192 ){
      old_pointer -= 192;
      for( count_x = 0 ; count_x < 8 ; count_x++ ){
	for( count_y = 0 ; count_y < 8 ; count_y++ ){
	  get_board( game_board, count_x, count_y ) = get_board( old_gameboard + old_pointer, count_x, count_y );
	}
      }
      
      redraw_board();
      
      old_pointer += 64;
    }
  }
}

win_game()
{
  char str[80];
  char str2[80];
  int cx, cy, black_t = 0, white_t = 0;

  for( cx = 0 ; cx < 8 ; cx++ ){
    for( cy = 0 ; cy < 8 ; cy++ ){
      if( get_board( game_board, cx, cy ) == 1 ){
	white_t++;
      }
      else if( get_board( game_board, cx, cy ) == 2 ){
	black_t++;
      }
    }
  }

  if( white_t > black_t ){
    strcpy( str, "White is the winner. " );
  }
  else if( white_t < black_t ){
    strcpy( str, "Black is the winner. " );
  }
  else{
    strcpy( str, "This game was a draw. " );
  }

  sprintf( str2, "White: %d, Black: %d", white_t, black_t );
  strcat( str, str2 );

  update_message( str );

  in_play = FALSE;
  update_playing();
}

void do_show_possible( Widget a, XtPointer b, XtPointer c )
{
  int cx, cy;

  for( cx = 0 ; cx < 8 ; cx++ ){
    for( cy = 0 ; cy < 8 ; cy++ ){
      if( possible_move_square( game_board, cx, cy, to_play ) ){
	XDrawLine( display, win, dgc, cx * (window_width / 8), cy * (window_height / 8),
		  (cx * (window_width / 8)) + (window_width / 8),
		  (cy * (window_height / 8)) + (window_height / 8) );
	XDrawLine( display, win, dgc, cx * (window_width / 8),
		  cy * (window_height / 8) + (window_height / 8),
		  (cx * (window_width / 8)) + (window_width / 8),
		  cy * (window_height / 8) );
      }
    }
  }

  showing_possible = TRUE;
}

void xsave_game( Widget a , XtPointer b, XtPointer c )
{
  if( in_play ){
    save_game( to_play );
  }
}

void redraw_board( void )
{
  XClearWindow( display, win );
  draw_board();
}

void draw_board( void )
{
  int cx, cy;
  XGCValues xgcv;
  int width_tile, x_add, y_add;

  xgcv.line_width = 0;
  XChangeGC( display, dgc, GCLineWidth, &xgcv );

  for( cx = 0 ; cx < 7 ; cx++ ){
    for( cy = 0 ; cy < 7 ; cy++ ){
      XDrawLine( display, win, dgc, (cx * (window_width / 8)) + (window_width / 8), 0,
		(cx * (window_width / 8)) + (window_width / 8), window_height );
    }
  }

  for( cx = 0 ; cx < 7 ; cx++ ){
    for( cy = 0 ; cy < 7 ; cy++ ){
      XDrawLine( display, win, dgc, 0, (cx * (window_height / 8)) + (window_height / 8),
		window_width, (cx * (window_height / 8)) + (window_height / 8) );
    }
  }
 
  xgcv.line_width = 4;
  XChangeGC( display, dgc, GCLineWidth, &xgcv );

  width_tile = MIN( (window_width / 8) - 8, (window_height / 8) - 8 );
  x_add = ((window_width / 8) - width_tile) / 2;
  y_add = ((window_height / 8) - width_tile) / 2;

  for( cx = 0 ; cx < 8 ; cx++ ){
    for( cy = 0 ; cy < 8 ; cy++ ){
      draw_tile( get_board( game_board, cx, cy ), cx, cy, width_tile, x_add, y_add, FALSE );
    }
  }

  /* Restore the linewidth, just in case */
  xgcv.line_width = 0;
  XChangeGC( display, dgc, GCLineWidth, &xgcv );
}

/*
 *  Handle all events, mouse press, window size changes etc...
 *
 */

void handle_event( Widget a, XtPointer b, XEvent *event )
{
  int tmp;

  switch( event->type ){
  case Expose:			/* The window is exposed */
    if( event->xexpose.count == 0 ){
      draw_board();
    }
    break;

  case ConfigureNotify:		/* Window size change */
    window_width = event->xconfigure.width;
    window_height = event->xconfigure.height;

    redraw_board();

    break;

  case ButtonPress:		/* Mouse button pressed */
    if( event->xbutton.button == 1 ){
      assert( (event->xbutton.x * 8) / window_width >= 0 &&
	     (event->xbutton.x * 8) / window_width <= 7 );
      assert( (event->xbutton.y * 8) / window_height >= 0 &&
	     (event->xbutton.y * 8) / window_height <= 7 );

      
      tmp = do_x_move( (event->xbutton.x * 8) / window_width, (event->xbutton.y * 8) / window_height, FALSE );
      while( tmp == 1 ){
	tmp = do_x_move( (event->xbutton.x * 8) / window_width, (event->xbutton.y * 8) / window_height, TRUE );
      }
    }
    break;

  }
}

void draw_tile( int t_type, int x, int y, int width_tile, int x_add, int y_add, int clear )
{
  if( clear ){
    XClearArea( display, win, (x * window_width / 8) + x_add,
	       (y * window_height / 8) + y_add,
	       width_tile, width_tile, FALSE );
  }    

  switch( t_type ){
  case 0:
    /* Actions when there is no tile here, 3D-board? */
    break;
    
  case 1:			/* White tile */
    XDrawArc( display, win, dgc, (x * (window_width / 8)) + x_add + 2,
	     (y * (window_height / 8)) + y_add + 2,
	     width_tile - 4, width_tile - 4, 0, 360 * 64 );
    break;
    
  case 2:			/* Black tile */
    XFillArc( display, win, dgc, (x * (window_width / 8)) + x_add,
	     (y * (window_height / 8)) + y_add,
	     width_tile, width_tile, 0, 360 * 64 );
    break;
  }

  XFlush( display );
}

void new_game( Widget a, XtPointer b, XtPointer c )
{
  int cx, cy;

  if( !in_play ){
    in_play = TRUE;
    
    init_board( 0 );
    to_play = 0;
    old_pointer = 0;

    redraw_board();

    if( play_computer[0] == 1 ){
      while( do_x_move( 0, 0, TRUE ) == 1 );
    }
  }
  else{
    in_play = FALSE;

    for( cx = 0 ; cx < 8 ; cx++ ){
      for( cy = 0 ; cy < 8 ; cy++ ){
	get_board( game_board, cx, cy ) = 0;
      }
    }

    redraw_board();
  }

  update_playing();
}

void select_level1( Widget a, XtPointer level, XtPointer c )
{
  if( in_play ){
    update_message( "Sorry, You can't change level while playing" );
  }
  else{
    if( *(int *)level == 0 ){
      play_computer[0] = 0;
      p1 = 0;
    }
    else{
      play_computer[0] = 1;
      p1 = 1;
      game_l[0] = *(int *)level - 1;
    }

    update_levels();
  }
}

void select_level2( Widget a, XtPointer level, XtPointer c )
{
  if( in_play ){
    update_message( "Sorry, You can't change level while playing" );
  }
  else{
    if( *(int *)level == 0 ){
      play_computer[1] = 0;
      p1 = 0;
    }
    else{
      play_computer[1] = 1;
      p2 = 1;
      game_l[1] = *(int *)level - 1;
    }

    update_levels();
  }
}

void update_message( char *str )
{
  Arg args[2];

  XtSetArg( args[0], XtNlabel, str );
  XtSetArg( args[1], XtNwidth, 560 );
  XtSetValues( message, args, TWO );
}

void update_playing( void )
{
  Arg args[1];

  if( !in_play ){
    XtSetArg( args[0], XtNlabel, "No game in progress" );
    XtSetValues( currentlyPlaying, args, ONE );
  }
  else{
    if( to_play == 0 ){
      XtSetArg( args[0], XtNlabel, "White to play" );
      XtSetValues( currentlyPlaying, args, ONE );
    }
    else{
      XtSetArg( args[0], XtNlabel, "Black to play" );
      XtSetValues( currentlyPlaying, args, ONE );
    }
  }
}

void update_levels( void )
{
  Arg args[1];
  char str[80];

  strcpy( str, "White: " );

  if( play_computer[0] == 0 ){
    strcat( str, "Human" );
  }
  else{
    strcat( str, level_name[game_l[0]] );
  }

  XtSetArg( args[0], XtNlabel, str );
  XtSetValues( levelMenuCommand1, args, ONE );

  strcpy( str, "Black: " );

  if( play_computer[1] == 0 ){
    strcat( str, "Human" );
  }
  else{
    strcat( str, level_name[game_l[1]] );
  }

  XtSetArg( args[0], XtNlabel, str );
  XtSetValues( levelMenuCommand2, args, ONE );
}
